package akhlys.fastdirsync;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

public class FileDifferencesTableModel extends AbstractTableModel {
	private static final long serialVersionUID = -4826207858228804111L;
	private List<RowClass> data = new ArrayList<>();
	private String[] header = {"", "File/Folder", "Difference", "Action"};

    public FileDifferencesTableModel() {
    	super();
    }
    
    public void setDifferences(List<FileDifference> differences, Path source, Path target) {
    	data.clear();
    	if (differences != null) {
	    	for (FileDifference diff : differences) {
	    		String relativePath = null;
	    		// the path is already normalized in DirSync but this is for reusability
	    		// toAbsolutePath is neccessary or java complains about comparing different types of paths
	    		if (diff.diffType != FileDiffType.DELETED) {
	    			relativePath = source.toAbsolutePath().relativize(diff.sourceFile.normalize()).toString();
	    		} else {
	    			// if deleted, source file does not exist
	    			relativePath = target.toAbsolutePath().normalize().relativize(diff.targetFile.normalize()).toString();
	    		}
	    		data.add(new RowClass(true, relativePath, diff));
	    	}
    	}
    	
    	fireTableDataChanged();
    }

    @Override
    public int getRowCount() {
        return data.size();
    }

    @Override
    public int getColumnCount() {
        return header.length;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
    	RowClass row = data.get(rowIndex);
    	switch (columnIndex) {
        case 0: return row.isSelected;
        case 1:	return row.relativePath;
        case 2: return row.fileDifference.diffType.toString();
        case 3:	return row.fileDifference.diffType.getRequiredAction();
        default:
            return "??";
    	}
    }
    
    public FileDifference getFileDifferenceAt(int rowIndex) {
    	return data.get(rowIndex).fileDifference;
    }
    
    public List<FileDifference> getAllFileDifferences() {
    	List<FileDifference> result = new ArrayList<>();
    	for (RowClass row : this.data) {
    		result.add(row.fileDifference);
    	}
    	return result;
    }
    

    public List<FileDifference> getSelectedFileDifferences() {
    	List<FileDifference> result = new ArrayList<>();
    	for (RowClass row : this.data) {
    		if (!row.isSelected) continue;
    		result.add(row.fileDifference);
    	}
    	return result;
    }

    @Override
    public String getColumnName(int index) {
        return header[index];
    }
    
    @Override
	public boolean isCellEditable(int row, int col) {
    	return col == 0;
    }
    
    @Override
	public Class<?> getColumnClass(int c) {
    	// sanity check for empty table
    	if (this.data == null || this.data.isEmpty()) return Object.class;
    	
    	if (c == 0) return Boolean.class;
    	return String.class;
    }
    
    @Override
    public void setValueAt(Object aValue, int row, int column) {
    	if (column != 0) return;
    	boolean b = Boolean.parseBoolean(aValue.toString());
    	RowClass rowcls = data.get(row);
    	if (rowcls.isSelected == b) return;
    	data.get(row).isSelected = b;
    	fireTableCellUpdated(row, column);
    }
    
    class RowClass {
    	public RowClass(boolean isSelected, String relativePath, FileDifference fileDifference) {
    		this.isSelected = isSelected;
    		this.relativePath = relativePath;
    		this.fileDifference = fileDifference;
    	}
    	public boolean isSelected;
    	public String relativePath;
    	public FileDifference fileDifference;
    }
}
